
// The package containing our stubs.
import Stock.*;
import Stock.Callback.*;

// HelloServer will use the naming service.
import org.omg.CosNaming.*;

// The package containing special exceptions thrown by the name service.
import org.omg.CosNaming.NamingContextPackage.*;

// All CORBA applications need these classes.
import org.omg.CORBA.*;

// For maps
import java.util.*;


public class StockServer2
{
  public static void main(String args[])
  {
    try{
    
      // Create and initialize the ORB
      ORB orb = ORB.init(args, null);

      // Create map
      Map cb_map = Collections.synchronizedMap(new HashMap());

      // Create the servant and register it with the ORB
      Notifying_QuoterServant quoterRef = new Notifying_QuoterServant(cb_map);
      orb.connect(quoterRef);
      
      // Get the root naming context
      org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
      NamingContext ncRef = NamingContextHelper.narrow(objRef);
      
      // Start monitoring thread
      (new Stock_Monitoring(cb_map)).start();

      // Bind the object reference in naming
      NameComponent nc = new NameComponent("NotifyingStock", "");
      NameComponent path[] = {nc};
      ncRef.rebind(path, quoterRef);
      
      // Wait for invocations from clients
      java.lang.Object sync = new java.lang.Object();
      synchronized(sync){
        sync.wait();
      }
      
    } catch(Exception e) {
        System.err.println("ERROR: " + e);
        e.printStackTrace(System.out);
      }  
  }
}


class Pair 
{
  public Pair(int first, Stock.Callback.Handler second) {
    _first = first;
    _second = second;
  }
  public int threshold() {return _first;}
  public Stock.Callback.Handler handler() {return _second;}

  private int _first;
  private Stock.Callback.Handler _second;
}

class Notifying_QuoterServant extends _Notifying_QuoterImplBase
{
  public Notifying_QuoterServant(Map cb) {
    super();
    map = cb;
  }

  public void register(String stock_name, int threshold, 
		Stock.Callback.Handler handler) 
       throws Stock.Invalid_Stock {
	 System.out.println("Registering " + stock_name);
	 map.put(stock_name, new Pair(threshold, handler));
  }	 

  private Map map;
}



class Stock_Monitoring extends Thread 
{
  public Stock_Monitoring(Map cb) {
    super();
    map = cb;
    System.out.println("Constructing monitor -- " + map.size() + " elements");
  }

  private Map map;

  public void run() {
    // Loop -- get new value
    // Verify if someone needs it
    // For now, just iterate over all clients
    for (;;) {
      try {
	sleep(1000);
	Iterator iter = map.entrySet().iterator();
	while (iter.hasNext()) {
	  Map.Entry e = (Map.Entry)iter.next();
	  String stock = (String)e.getKey();
	  Pair p = (Pair)e.getValue();
	  System.out.println("Pushing " + stock + " " + p.threshold());
	  try {
	    p.handler().push(new Stock.Callback.Info(stock,
						     p.threshold()));
	  } catch (Exception e1) {
	    System.err.println("Client for " + stock + " has unregistered");
	    iter.remove();
	  }
	}
      } catch(Exception e2) {
	System.err.println("ERROR: " + e2);
	e2.printStackTrace(System.out);
      }  
    }
  }
}
